///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CMIMETYPES_CPP
#define _CMIMETYPES_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>

#include "CLocks.H"

#ifdef _DEBUG
#define CLocksAssert(sExpression, sMsg) \
	if(sExpression)\
	{ \
		char sAssert[2048]; \
		sprintf_s(sAssert, sizeof(sAssert), \
			"%s\r\n\r\n" \
			"Procedure: %s\r\n" \
			"Source File: %s\r\n" \
			"Build Date/Time: %s (%s)\r\n" \
			"Expression: \"%s\"\r\n" \
			"Line: %d\r\n\r\n", \
			sMsg, __FUNCTION__, __FILE__, __DATE__, __TIME__, #sExpression, __LINE__); \
		MessageBox(NULL, sAssert, "CMemPool Exception", MB_ICONSTOP | MB_SYSTEMMODAL); \
		__debugbreak(); \
	}
#else
	#define CLocksAssert(sExpression, sMsg)
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CLocks::CLocks(void)
{
	InitializeCriticalSection(&CS);
	this->iRegularLockCount = 0;
	this->iExclusiveLockCount = 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CLocks::~CLocks(void)
{
	DeleteCriticalSection(&CS);
	this->iRegularLockCount = 0;
	this->iExclusiveLockCount = 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CLocks::RegularLocks(void)
{
	return this->iRegularLockCount;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CLocks::ExclusiveLocks(void)
{
	return this->iExclusiveLockCount;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CLocks::Locks(void)
{
	return this->iRegularLockCount + this->iExclusiveLockCount;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CLocks::Lock(void)
{
	if(this->iExclusiveLockCount > 0)
	{
		printf("");
	}
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount < 0, "Regular lock count must be greater than zero.");
	this->iRegularLockCount++;
	LeaveCriticalSection(&this->CS);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CLocks::UnLock(char *sValue)
{
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount <= 0, "Regular lock not held.");
	this->iRegularLockCount--;
	LeaveCriticalSection(&this->CS);
	return sValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CLocks::UnLock(double bValue)
{
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount <= 0, "Regular lock not held.");
	this->iRegularLockCount--;
	LeaveCriticalSection(&this->CS);
	return bValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

__int64 CLocks::UnLock(__int64 i64Value)
{
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount <= 0, "Regular lock not held.");
	this->iRegularLockCount--;
	LeaveCriticalSection(&this->CS);
	return i64Value;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned __int64 CLocks::UnLock(unsigned __int64 i64Value)
{
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount <= 0, "Regular lock not held.");
	this->iRegularLockCount--;
	LeaveCriticalSection(&this->CS);
	return i64Value;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int CLocks::UnLock(unsigned int iValue)
{
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount <= 0, "Regular lock not held.");
	this->iRegularLockCount--;
	LeaveCriticalSection(&this->CS);
	return iValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CLocks::UnLock(int iValue)
{
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount <= 0, "Regular lock not held.");
	this->iRegularLockCount--;
	LeaveCriticalSection(&this->CS);
	return iValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CLocks::UnLock(bool bValue)
{
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount <= 0, "Regular lock not held.");
	this->iRegularLockCount--;
	LeaveCriticalSection(&this->CS);
	return bValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CLocks::UnLock(void)
{
	EnterCriticalSection(&this->CS);
	CLocksAssert(this->iRegularLockCount <= 0, "Regular lock not held.");
	this->iRegularLockCount--;
	LeaveCriticalSection(&this->CS);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CLocks::LockExclusive(void)
{
	while(true) //Wait for all other locks to clear.
	{
		EnterCriticalSection(&this->CS); //Lock the resources.
		CLocksAssert(this->iExclusiveLockCount < 0, "Exclusive lock count must be greater than zero.");

		//If we already have an exclusive lock, then we're good. Otherwise we
		//	need to wait for the regular lock count to become zero.
		//Keep in mind that this class allows regular locks by the same thread
		//	that owns the exclusive lock.
		if(this->iRegularLockCount == 0 || this->iExclusiveLockCount > 0)
		{
			this->iExclusiveLockCount++;
			break;
		}
		LeaveCriticalSection(&this->CS); //Unlock the resources.

		Sleep(1); //Give other threads time to do their jobs.
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CLocks::UnLockExclusive(char *sValue)
{
	CLocksAssert(this->iRegularLockCount != 0, "Regular locks still held.");
	CLocksAssert(this->iExclusiveLockCount <= 0, "Exclusive lock not held.");
	this->iExclusiveLockCount--;
	LeaveCriticalSection(&this->CS);
	return sValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CLocks::UnLockExclusive(int iValue)
{
	CLocksAssert(this->iRegularLockCount != 0, "Regular locks still held.");
	CLocksAssert(this->iExclusiveLockCount <= 0, "Exclusive lock not held.");
	this->iExclusiveLockCount--;
	LeaveCriticalSection(&this->CS);
	return iValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CLocks::UnLockExclusive(bool bValue)
{
	CLocksAssert(this->iRegularLockCount != 0, "Regular locks still held.");
	CLocksAssert(this->iExclusiveLockCount <= 0, "Exclusive lock not held.");
	this->iExclusiveLockCount--;
	LeaveCriticalSection(&this->CS);
	return bValue;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CLocks::UnLockExclusive(void)
{
	CLocksAssert(this->iRegularLockCount != 0, "Regular locks still held.");
	CLocksAssert(this->iExclusiveLockCount <= 0, "Exclusive lock not held.");
	this->iExclusiveLockCount--;
	LeaveCriticalSection(&this->CS);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
